@@ -20,8 +20,9 @@
 package server
 
 import (
+	"context"
+
 	"github.com/containerd/containerd/errdefs"
-	"golang.org/x/net/context"
 	runtime "k8s.io/cri-api/pkg/apis/runtime/v1alpha2"
 )
 
