@@ -26,21 +26,24 @@ import (
 	"net/http"
 	"net/url"
 	"time"
+
+	"github.com/elastic/elastic-agent-libs/upgrade/details"
 )
 
 const (
-	fleetAgentPoliciesAPI     = "/api/fleet/agent_policies"
-	fleetAgentPolicyAPI       = "/api/fleet/agent_policies/%s"
-	fleetAgentsDeleteAPI      = "/api/fleet/agent_policies/delete"
-	fleetEnrollmentAPIKeysAPI = "/api/fleet/enrollment_api_keys" //nolint:gosec // no API key being leaked here
-	fleetAgentsAPI            = "/api/fleet/agents"
-	fleetAgentAPI             = "/api/fleet/agents/%s"
-	fleetUnEnrollAgentAPI     = "/api/fleet/agents/%s/unenroll"
-	fleetUpgradeAgentAPI      = "/api/fleet/agents/%s/upgrade"
-	fleetFleetServerHostsAPI  = "/api/fleet/fleet_server_hosts"
-	fleetFleetServerHostAPI   = "/api/fleet/fleet_server_hosts/%s"
-	fleetPackagePoliciesAPI   = "/api/fleet/package_policies"
-	fleetUninstallTokensAPI   = "/api/fleet/uninstall_tokens" //nolint:gosec // NOT the "Potential hardcoded credentials"
+	fleetAgentAPI                = "/api/fleet/agents/%s"
+	fleetAgentPoliciesAPI        = "/api/fleet/agent_policies"
+	fleetAgentPolicyAPI          = "/api/fleet/agent_policies/%s"
+	fleetAgentsAPI               = "/api/fleet/agents"
+	fleetAgentsDeleteAPI         = "/api/fleet/agent_policies/delete"
+	fleetEnrollmentAPIKeysAPI    = "/api/fleet/enrollment_api_keys" //nolint:gosec // no API key being leaked here
+	fleetFleetServerHostAPI      = "/api/fleet/fleet_server_hosts/%s"
+	fleetFleetServerHostsAPI     = "/api/fleet/fleet_server_hosts"
+	fleetPackagePoliciesAPI      = "/api/fleet/package_policies"
+	fleetUnEnrollAgentAPI        = "/api/fleet/agents/%s/unenroll"
+	fleetUninstallTokensAPI      = "/api/fleet/uninstall_tokens" //nolint:gosec // NOT the "Potential hardcoded credentials"
+	fleetUpgradeAgentAPI         = "/api/fleet/agents/%s/upgrade"
+	fleetAgentDownloadSourcesAPI = "/api/fleet/agent_download_sources"
 )
 
 //
@@ -140,6 +143,68 @@ func (client *Client) CreatePolicy(ctx context.Context, request AgentPolicy) (r
 	return polResp.Item, err
 }
 
+type DownloadSource struct {
+	Name      string      `json:"name"`
+	Host      string      `json:"host"`
+	IsDefault bool        `json:"is_default"`
+	ProxyID   interface{} `json:"proxy_id"`
+}
+
+type DownloadSourceResponse struct {
+	Item struct {
+		ID        string `json:"id"`
+		Name      string `json:"name"`
+		Host      string `json:"host"`
+		IsDefault bool   `json:"is_default"`
+		ProxyID   string `json:"proxy_id"`
+	} `json:"item"`
+}
+
+func (client *Client) CreateDownloadSource(ctx context.Context, source DownloadSource) (DownloadSourceResponse, error) {
+	reqBody, err := json.Marshal(source)
+	if err != nil {
+		return DownloadSourceResponse{},
+			fmt.Errorf("unable to marshal DownloadSource into JSON: %w", err)
+	}
+
+	resp, err := client.Connection.SendWithContext(
+		ctx,
+		http.MethodPost,
+		fleetAgentDownloadSourcesAPI,
+		nil,
+		nil,
+		bytes.NewReader(reqBody))
+	if err != nil {
+		return DownloadSourceResponse{},
+			fmt.Errorf("error calling Agent Binary Download Sources API: %w", err)
+	}
+	defer resp.Body.Close()
+
+	if resp.StatusCode != http.StatusOK {
+		var respBody string
+		if bs, err := io.ReadAll(resp.Body); err != nil {
+			respBody = "could not read response body"
+		} else {
+			respBody = string(bs)
+		}
+
+		client.log.Errorw(
+			"could not create download source, kibana returned "+resp.Status,
+			"http.response.body.content", respBody)
+		return DownloadSourceResponse{},
+			fmt.Errorf("could not create download source, kibana returned %s. response body: %s: %w",
+				resp.Status, respBody, err)
+	}
+
+	body := DownloadSourceResponse{}
+	if err = json.NewDecoder(resp.Body).Decode(&body); err != nil {
+		return DownloadSourceResponse{},
+			fmt.Errorf("failed parsing download source response: %w", err)
+	}
+
+	return body, nil
+}
+
 // GetPolicy returns the requested ID
 func (client *Client) GetPolicy(ctx context.Context, id string) (r PolicyResponse, err error) {
 	apiURL := fmt.Sprintf(fleetAgentPolicyAPI, id)
@@ -256,8 +321,22 @@ type AgentCommon struct {
 			Hostname string `json:"hostname"`
 		} `json:"host"`
 	} `json:"local_metadata"`
-	PolicyID       string `json:"policy_id"`
-	PolicyRevision int    `json:"policy_revision"`
+	PolicyID       string               `json:"policy_id"`
+	PolicyRevision int                  `json:"policy_revision"`
+	UpgradeDetails *AgentUpgradeDetails `json:"upgrade_details"`
+}
+
+type AgentUpgradeDetails struct {
+	TargetVersion string `json:"target_version"`
+	State         string `json:"state"`
+	ActionID      string `json:"action_id"`
+	Metadata      struct {
+		ScheduledAt     *time.Time           `json:"scheduled_at"`
+		DownloadPercent float64              `json:"download_percent"`
+		DownloadRate    details.DownloadRate `json:"download_rate"`
+		FailedState     string               `json:"failed_state"`
+		ErrorMsg        string               `json:"error_msg"`
+	} `json:"metadata"`
 }
 
 // AgentExisting is the data structure for an existing agent
